//
//  VMLocationsManager.h
//  VMobSDK
//
//  Created by vmob.co on 6/3/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMRequestManager.h"
#import <CoreLocation/CoreLocation.h>

/**
 `VMLocationsManager` is a subclass of `VMRequestManager` providing methods for getting the states and cities lists. This API should be used to select a location if gps/network location is not available on mobile device.
 */
@interface VMLocationsManager : VMRequestManager

/**
 Current location.
 */
@property (readonly, nonatomic, strong) CLLocation *currentLocation;

/**
 Read-only variable showing if location is being updated: YES if the location is being updated, else NO.
 */
@property (nonatomic, readonly) BOOL isUpdatingLocation;

/**
 Read-only variable showing the Time interval for updating location.
 */
@property (nonatomic, readonly) NSTimeInterval updateLocationTimeInterval;

/**
 Returns the shared locations manager object for the system.
 
 @return The systemwide locations manager.
 */
+ (instancetype) sharedInstance;

/**
 Provides a list of all States.
 
 @param completionBlock The block to be executed on the completion of a successful request. This block has no return value and takes one argument: the object constructed from the response data of the request.
 @param errorBlock The block to be executed on the completion of an unsuccessful request. This block has no return value and takes one argument: the error that occurred during the request.
 
 @return A new request operation.
 */
- (VMHTTPRequestOperation *) statesListWithCompletionBlock: (ObjectBlock) completionBlock
												 failBlock: (ErrorBlock) errorBlock;

/**
 Provides a list of all Cities.
 
 @param completionBlock The block to be executed on the completion of a successful request. This block has no return value and takes one argument: the object constructed from the response data of the request.
 @param errorBlock The block to be executed on the completion of an unsuccessful request. This block has no return value and takes one argument: the error that occurred during the request.
 
 @return A new request operation.
 */
- (VMHTTPRequestOperation *) citiesListWithCompletionBlock: (ObjectBlock) completionBlock
												 failBlock: (ErrorBlock) errorBlock;
#pragma mark - Check Location

/**
 Stop updating locations.
 */
- (void) stopLocation;

/**
 Start updating locations.
 */
- (void) startLocation;

/**
 Start updating locations.
 
 @param updateLocationBlock block called on successful location update
 @param errorBlock block called on errorreceived when updating location
 */
- (void) startLocationWithCompletionBlock: (ObjectBlock) updateLocationBlock
								failBlock: (ErrorBlock) errorBlock;

/**
 Set block called on successful location update
 
 @param updateLocationBlock block called on successful location update
 */
- (void) setUpdateLocation: (ObjectBlock) updateLocationBlock;

/**
 Set block called on errorreceived when updating location
 
 @param errorBlock block called on errorreceived when updating location
 */
- (void) setUpdateErrorBlock: (ErrorBlock) errorBlock;

/**
 Set block called on adding current location to activity batch
 
 @param updateActivityLocationBlock block called on adding current location to activity batch
 */
- (void) setUpdateActivityLocation: (ObjectBlock) updateActivityLocationBlock;

@end
