//
//  ConsumerViewController.m
//  SampleVMSDK
//
//  Created by vmob.co on 5/30/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "ConsumerViewController.h"
#import "ManagerConsumerViewController.h"
#import <FacebookSDK/FacebookSDK.h>
#import "VMConsumerManager.h"
#import "AppDelegate.h"

@interface ConsumerViewController ()<UIAlertViewDelegate>
@property (strong, nonatomic) IBOutlet UIBarButtonItem *cancelBarButtonItem;
@property (nonatomic, weak) VMHTTPRequestOperation *currentOperation;
@property (weak, nonatomic) IBOutlet UILabel *tokenPushNotificationLabel;
@property (strong, nonatomic) IBOutlet UITextView *tokenTextView;
@end

@implementation ConsumerViewController

- (void)viewDidLoad {
	[super viewDidLoad];
	self.title = @"Consumer API";
	self.navigationItem.rightBarButtonItem = self.cancelBarButtonItem;
	self.tokenPushNotificationLabel.text = ((AppDelegate *)[UIApplication sharedApplication].delegate).deviceToken;
    self.tokenTextView.text = ((AppDelegate *)[UIApplication sharedApplication].delegate).deviceToken;
}

#pragma mark - Action Methods
- (IBAction)copyButtonAction:(id)sender {
    
    UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
    pasteboard.string = ((AppDelegate *)[UIApplication sharedApplication].delegate).deviceToken;
}

- (IBAction)registerEmailButtonAction:(id)sender {
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"RegisterEmail" message:@"email" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
	[alertView setAlertViewStyle:UIAlertViewStylePlainTextInput];
	alertView.tag = 0;
	[alertView show];
}

- (IBAction)registerMobileButtonAction:(id)sender {
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"RegisterMobile" message:@"Mobile" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
	[alertView setAlertViewStyle:UIAlertViewStylePlainTextInput];
	alertView.tag = 1;
	[alertView show];
}

- (IBAction)registerAnonymousButtonAction:(id)sender {
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"RegisterAnonymous" message:@"Anonymous" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
	[alertView setAlertViewStyle:UIAlertViewStylePlainTextInput];
	alertView.tag = 2;
	[alertView show];
}

- (IBAction)loginButtonAction:(UIButton *)sender {
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Login" message:@"Mobile" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
	[alertView setAlertViewStyle:UIAlertViewStyleLoginAndPasswordInput];
	alertView.tag = 3;
	[alertView show];
}

- (IBAction)loginFacebookButtonAction:(UIButton *)sender {
	[self loginFacebook];
}

//example canceling request
- (IBAction)cancelRequestButtonAction:(id)sender {
	[self.currentOperation cancel];
}

- (IBAction)managerButtonAction:(id)sender {
	[self.navigationController pushViewController:[ManagerConsumerViewController new] animated:YES];
}

- (IBAction)crossReferenceButtonAction:(id)sender {
	[self createCrossReference];
}

- (IBAction) facebookDowngrade:(id)sender
{
    //[NSString stringWithString:FBSession.activeSession.accessToken];
    
    [[VMConsumerManager sharedInstance] changeConsumerFacebookAccountToAnonymousWithFBSessionToken:[VMConsumerManager sharedInstance].userToken completionBlock:^(NSDictionary *dict){
        NSLog(@"%@", dict);
        [[[UIAlertView alloc] initWithTitle:@"Success" message:nil delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];
    } failBlock:^(NSError *error){
        NSLog(@"%@", error);
    }];
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	if (buttonIndex != alertView.cancelButtonIndex) {
		NSString *text = [alertView textFieldAtIndex:0].text;
		switch (alertView.tag) {
			case 0:
				[self registerEmail:text];
				break;
			case 1:
				[self registerMobilePhoneNumber:text];
				break;
			case 2:
				[self registerAnonymousWithUserName:text];
				break;
			case 3:
				[self loginWithUserName:text password:[alertView textFieldAtIndex:1].text];
			default:
				break;
		}
		
	}
}

#pragma mark - Send Method

- (void) createCrossReference{
	[[VMConsumerManager sharedInstance] createCrossReferenceWithPushNotification:((AppDelegate *)[UIApplication sharedApplication].delegate).deviceToken
																	   userToken:[VMConsumerManager sharedInstance].userToken
																 completionBlock:^(NSDictionary *description) {
																	 [[[UIAlertView alloc] initWithTitle:@"Completion createCrossReference" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																 } failBlock:^(NSError *error) {
																	 [[[UIAlertView alloc] initWithTitle:@"Fail createCrossReference" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																 }];
}

- (void) registerEmail: (NSString *) email {
	self.currentOperation = [[VMConsumerManager sharedInstance] registerConsumerWithEmail:email
																		  userName:email
																		  password:@"qwe123"
																		 firstName:@"First"
																		  lastName:@"Last"
																		  fullName:@"Full"
																			gender:VMMale
																	   dateOfBirth:[NSDate date]
																			  city:@"2"
																   completionBlock:^(NSDictionary *description) {
																	   [[[UIAlertView alloc] initWithTitle:@"Completion email" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																   }
																		 failBlock:^(NSError *error) {
																			 [[[UIAlertView alloc] initWithTitle:@"Fail email" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																		 }];
}

- (void) registerMobilePhoneNumber: (NSString *) mobilePhoneNumber {
	self.currentOperation = [[VMConsumerManager sharedInstance] registerConsumerWithMobilePhone:mobilePhoneNumber
																				userName:mobilePhoneNumber
																				password:@"qwe123"
																			   firstName:@"First"
																				lastName:@"Last"
																				fullName:@"Full"
																				  gender:VMMale
																			 dateOfBirth:[NSDate date]
																					city:@"2"
																		 completionBlock:^(NSDictionary *description) {
																			 [[[UIAlertView alloc] initWithTitle:@"Completion email" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																		 }
																			   failBlock:^(NSError *error) {
																				   [[[UIAlertView alloc] initWithTitle:@"Fail email" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																			   }];
}

- (void) registerAnonymousWithUserName: (NSString *) userName {
	self.currentOperation = [[VMConsumerManager sharedInstance] registerConsumerWithAnonymousUserName: userName
																					  password:@"qwe123"
																			   completionBlock:^(NSDictionary *description) {
																				   [[[UIAlertView alloc] initWithTitle:@"Completion registerConsumerWithAnonymous" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																			   }
																					 failBlock:^(NSError *error) {
																						 [[[UIAlertView alloc] initWithTitle:@"Fail registerConsumerWithAnonymous" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																					 }];
}

- (void) loginWithUserName: (NSString *) userName password: (NSString *) password {
	self.currentOperation = [[VMConsumerManager sharedInstance] loginConsumerWithUserName:userName
																		  password:password
																   completionBlock:^(NSDictionary *description) {
																	   [[[UIAlertView alloc] initWithTitle:@"Completion loginConsumerWithUserName" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																   }
																		 failBlock:^(NSError *error) {
																			 [[[UIAlertView alloc] initWithTitle:@"Fail loginConsumerWithUserName" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																		 }];
}

- (void) loginWithSocialToken: (NSString *) token {
	self.currentOperation = [[VMConsumerManager sharedInstance] loginConsumerWithSocialToken:token
																		   socialType:VMFacebook
																   completionBlock:^(NSDictionary *description) {
                                                                       NSLog(@"Facebook access token %@", FBSession.activeSession.accessToken);
																	   [[[UIAlertView alloc] initWithTitle:@"Completion Facebook Login" message:[NSString stringWithFormat:@"%@",description] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																   }
																		 failBlock:^(NSError *error) {
                                                                             NSLog(@"Facebook access token %@", FBSession.activeSession.accessToken);
																			 [[[UIAlertView alloc] initWithTitle:@"Fail Facebook Login" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
																		 }];
}

#pragma mark - Facebook Methods

- (void) loginFacebook {
    NSLog(@"Facebook access token %@", FBSession.activeSession.accessToken);
	if (FBSession.activeSession.isOpen) {
		[self loginWithSocialToken:[NSString stringWithString:FBSession.activeSession.accessToken]];
		return;
	}
	FBSession.activeSession = [[FBSession alloc] init];
	if (FBSession.activeSession.state != FB_SESSIONSTATEOPENBIT) {
		[FBSession.activeSession openWithCompletionHandler:^(FBSession *session,
															 FBSessionState status,
															 NSError *error) {
			if (error) {
				[[[UIAlertView alloc] initWithTitle:@"Facebook Auth" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
			}
			else if(status != FBSessionStateClosedLoginFailed && status!= FBSessionStateClosed) {
				[FBSession setActiveSession:session];
				[self loginWithSocialToken:[NSString stringWithString:FBSession.activeSession.accessToken]];
			}
			else {
				[[[UIAlertView alloc] initWithTitle:@"Facebook Auth" message:@"Cancel login" delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil] show];
			}
		}];
	}
	else {
		[self loginWithSocialToken:[NSString stringWithString:FBSession.activeSession.accessToken]];
	}
}

- (void)viewDidUnload {
    [self setTokenTextView:nil];
    [super viewDidUnload];
}
@end