//
//  VMActivity.m
//  VMobSDK
//
//  Created by vmob.co on 7/3/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMActivity.h"
#import "NSDictionary+VMValidation.h"
#import "VMUtils.h"
#import "NSString+VMToolkitAdditions.h"

@implementation VMActivity

- (void) updatePropertyWithDictionary: (NSDictionary *) dictionary {
	self.actionTypeCode = [dictionary objectForKey:@"actionTypeCode"];
	self.actionCode = [dictionary objectForKey:@"actionCode"];
	self.value1 = [dictionary objectForKey:@"actionValue1"];
	self.value2 = [dictionary objectForKey:@"actionValue2"];
	self.value3 = [dictionary objectForKey:@"actionValue3"];
	self.redemptionId = [dictionary objectForKey:@"redemptionId"];
	self.latitude = [dictionary objectForKey:@"latitude"];
	self.longitude = [dictionary objectForKey:@"longitude"];
	self.ownerId = [dictionary objectForKey:@"ownerId"];
	self.macId = [dictionary objectForKey:@"macId"];
	self.merchantId = [dictionary objectForKey:@"merchantId"];
	self.campaignId = [dictionary objectForKey:@"campaignId"];
	self.itemId = [dictionary objectForKey:@"itemId"];
	self.venueId = [dictionary objectForKey:@"venueId"];
    self.isGiftable = [[dictionary objectForKey:@"isGiftable"] boolValue];
    self.isAGift = [[dictionary objectForKey:@"isAGift"] boolValue];
    self.GiftedBy = [dictionary objectForKey:@"giftedBy"];
    self.GiftedCopy =[dictionary objectForKey:@"giftedCopy"];
}

- (VMActivity *) initWithDictionary: (NSDictionary *) dictionary {
	if (self = [super init]) {
		[self updatePropertyWithDictionary:dictionary];
	}
	return self;
}

#pragma mark - Comparison Methods

+ (BOOL) compareFirstDict: (NSDictionary *) first secondDict: (NSDictionary *) second {
	NSDate *date1 = [VMUtils dateUTCString:[first objectForKey:@"sourceActivityTime"]];
	NSDate *date2 = [VMUtils dateUTCString:[second objectForKey:@"sourceActivityTime"]];
	return [date1 compare:date2];
}

#pragma mark - Validate

+ (BOOL) validateSourceActivityTime: (NSDate *) date {
	NSTimeInterval timeInterval = [[NSDate date] timeIntervalSinceDate:date];
	return (timeInterval < [VMUtils activitiesExpireAgeInSeconds]);
}

+ (BOOL) validateActivityWithDictionary: (NSDictionary *) dictionary {
	NSDate *date = [VMUtils dateUTCString:[dictionary objectForKey:@"sourceActivityTime"]];
	return [self validateSourceActivityTime:date];
}

#pragma mark - Universal Dictionary

+ (NSMutableDictionary *) mutableDictionaryWithActionTypeCode: (VMActionTypeCode) actionTypeCode
												   actionCode: (NSString *) actionCode
													   value1: (NSString *) value1
													   value2: (NSString *) value2
													   value3: (NSString *) value3
												 redemptionId: (NSString *) redemptionId
													 latitude: (NSString *) latitude
													longitude: (NSString *) longitude
													  ownerId: (NSString *) ownerId
														macId: (NSString *) macId
												   merchantId: (NSString *) merchantId
												   campaignId: (NSString *) campaignId
													   itemId: (NSString *) itemId
													 itemCode: (NSString *) itemCode
													  venueId: (NSString *) venueId {
	NSMutableDictionary *dict = [NSMutableDictionary dictionary];
	[dict setObjectOrDoNothing:[NSString stringWithUUID] forKey:@"id"];
	[dict setObjectOrDoNothing:[VMUtils UTCFormateDate:[NSDate date]] forKey:@"sourceActivityTime"];
	[dict setObjectOrDoNothing:[VMUtils offsetUTC] forKey:@"sourceActivityTimeZoneOffset"];
	
	[dict setObjectOrDoNothing:[self stringWithActionTypeCode:actionTypeCode] forKey:@"actionTypeCode"];
	[dict setObjectOrDoNothing:actionCode forKey:@"actionCode"];
	[dict setObjectOrDoNothing:value1 forKey:@"actionValue1"];
	[dict setObjectOrDoNothing:value2 forKey:@"actionValue2"];
	[dict setObjectOrDoNothing:value3 forKey:@"actionValue3"];
	[dict setObjectOrDoNothing:redemptionId forKey:@"redemptionId"];
	[dict setObjectOrDoNothing:latitude forKey:@"latitude"];
	[dict setObjectOrDoNothing:longitude forKey:@"longitude"];
	[dict setObjectOrDoNothing:ownerId forKey:@"ownerId"];
	[dict setObjectOrDoNothing:macId forKey:@"macId"];
	[dict setObjectOrDoNothing:merchantId forKey:@"merchantId"];
	[dict setObjectOrDoNothing:campaignId forKey:@"campaignId"];
	[dict setObjectOrDoNothing:itemId forKey:@"itemId"];
	[dict setObjectOrDoNothing:itemCode forKey:@"itemCode"];
	[dict setObjectOrDoNothing:venueId forKey:@"venueId"];
	return dict;
}

#pragma mark - Button Click

+ (NSMutableDictionary *) dictionaryButtonClickAnonymousWithButtonId: (NSString *) buttonId
														  merchantId: (NSString *) merchantId
															 venueId: (NSString *) venueId
															  itemId: (NSString *) itemId
															itemCode: (NSString *) itemCode {
	NSMutableDictionary *dict = [self dictionaryButtonClickWithButtonId:buttonId merchantId:merchantId venueId:venueId itemId:itemId itemCode:itemCode];
	[dict setObjectOrDoNothing:[self stringWithActionTypeCode:VMButtonClickAnonym] forKey:@"actionTypeCode"];
	return dict;
}

+ (NSMutableDictionary *) dictionaryButtonClickWithButtonId: (NSString *) buttonId
												 merchantId: (NSString *) merchantId
													venueId: (NSString *) venueId
													 itemId: (NSString *) itemId
												   itemCode: (NSString *) itemCode {
	
	return [self mutableDictionaryWithActionTypeCode:VMButtonClick
										  actionCode:buttonId
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:merchantId
										  campaignId:nil
											  itemId:itemId
											itemCode:itemCode
											 venueId:venueId];
}

#pragma mark - Location Checkin

+ (NSMutableDictionary *) dictionaryLocationCheckinWithTypeGPS: (VMTypeGPS) typeGPS
											  locationAccuracy: (NSString *) locationAccuracy
												 currentCourse: (NSString *) currentCourse
														 speed: (NSString *) speed
													  altitude: (NSString *) altitude
													  latitude: (NSString *) latitude
													 longitude: (NSString *) longitude {
	return [self mutableDictionaryWithActionTypeCode:VMLocationCheckin
										  actionCode:nil
											  value1:[self stringWithTypeGPS:typeGPS]
											  value2:locationAccuracy
											  value3:[NSString stringWithFormat:@"%@;%@;%@", currentCourse, speed, altitude]
										redemptionId:nil
											latitude:latitude
										   longitude:longitude
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}

#pragma mark - AppPageImpression

+ (NSMutableDictionary *) dictionaryAppPageImpressionAnonymWithPageId: (NSString *) pageId
														   merchantId: (NSString *) merchantId
															  venueId: (NSString *) venueId
															   itemId: (NSString *) itemId
															 itemCode: (NSString *) itemCode {
	
	NSMutableDictionary *dict = [self dictionaryAppPageImpressionWithPageId:pageId
																 merchantId:merchantId
																	venueId:venueId
																	 itemId:itemId
																   itemCode:itemCode];
	[dict setObjectOrDoNothing:[self stringWithActionTypeCode:VMAppPageImpressionAnonym] forKey:@"actionTypeCode"];
	return dict;
}

+ (NSMutableDictionary *) dictionaryAppPageImpressionWithPageId: (NSString *) pageId
													 merchantId: (NSString *) merchantId
														venueId: (NSString *) venueId
														 itemId: (NSString *) itemId
													   itemCode: (NSString *) itemCode {
	return [self mutableDictionaryWithActionTypeCode:VMAppPageImpression
										  actionCode:pageId
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:merchantId
										  campaignId:nil
											  itemId:itemId
											itemCode:itemCode
											 venueId:venueId];
}

#pragma mark - Offer

+ (NSMutableDictionary *) dictionaryOfferImpressionWithPlacementType: (NSString *) placementType
															 offerId: (NSString *) offerId {
	return [self mutableDictionaryWithActionTypeCode:VMOfferImpression
										  actionCode:placementType
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:offerId
											itemCode:@"O"
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryOfferRedemptionWithPlacementType: (NSString *) placementType
													 paymentTypeCode: (NSString *) paymentTypeCode
														   paymentId: (NSString *) paymentId
													   paymentAmount: (NSString *) paymentAmount
													   redeemedOffer: (NSString *) redeemedOffer
															 offerId: (NSString *) offerId
															 venueId: (NSString *) venueId {
	return [self mutableDictionaryWithActionTypeCode:VMOfferRedemption
										  actionCode:placementType
											  value1:paymentTypeCode
											  value2:paymentId
											  value3:paymentAmount
										redemptionId:redeemedOffer
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:offerId
											itemCode:@"O"
											 venueId:venueId];
}

+ (NSMutableDictionary *) dictionaryOfferClickThroughWithPlacementType: (NSString *) placementType
															   offerId: (NSString *) offerId {
	return [self mutableDictionaryWithActionTypeCode:VMOfferClickThrough
										  actionCode:placementType
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:offerId
											itemCode:@"O"
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryOfferInstoreRedemptionWithPlacementType: (NSString *) placementType
															  redeemedOffer: (NSString *) redeemedOffer
																	offerId: (NSString *) offerId
																	venueId: (NSString *) venueId {
	return [self mutableDictionaryWithActionTypeCode:VMOfferInstoreRedemption
										  actionCode:placementType
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:redeemedOffer
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:offerId
											itemCode:@"O"
											 venueId:venueId];
}

#pragma mark - Application

+ (NSMutableDictionary *) dictionaryAppStartup {
	return [self mutableDictionaryWithActionTypeCode:VMAppStartup
										  actionCode:nil
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryAppStartupAnonym {
	return [self mutableDictionaryWithActionTypeCode:VMAppStartupAnonym
										  actionCode:nil
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryAppInstall {
	return [self mutableDictionaryWithActionTypeCode:VMAppInstall
										  actionCode:nil
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}



#pragma mark - Social

+ (NSMutableDictionary *) dictionaryRegistrationWithRegistrationType: (VMActivitySocialType) socialType {
	return [self mutableDictionaryWithActionTypeCode:VMRegistration
										  actionCode:@"Mobile"
											  value1:[self stringWithActivitySocialType:socialType]
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryLoginWithLoginType: (VMActivitySocialType) socialType {
	return [self mutableDictionaryWithActionTypeCode:VMLogin
										  actionCode:[self stringWithActivitySocialType:socialType]
											  value1:nil
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:nil
											itemCode:nil
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryOfferShareWithSocialType: (VMShareSocialType) socialType
													 offerId: (NSString *) offerId {
	NSMutableDictionary *dict =  [self mutableDictionaryWithActionTypeCode:VMOfferShare
																actionCode:[self stringWithShareSocialType:socialType]
																	value1:nil
																	value2:nil
																	value3:nil
															  redemptionId:nil
																  latitude:nil
																 longitude:nil
																   ownerId:nil
																	 macId:nil
																merchantId:nil
																campaignId:nil
																	itemId:offerId
																  itemCode:nil
																   venueId:nil];
	if (offerId) {
		[dict setObjectOrDoNothing:@"O" forKey:@"itemCode"];
	}
	return dict;
}

#pragma mark - Advertisement

+ (NSMutableDictionary *) dictionaryAdvertisementImpressionWithPlacementCode: (NSString *) placementCode
																 channelCode: (NSString *) channelCode
															 advertisementId: (NSString *) advertisementId {
	return [self mutableDictionaryWithActionTypeCode:VMAdvertisementImpression
										  actionCode:placementCode
											  value1:channelCode
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:advertisementId
											itemCode:@"A"
											 venueId:nil];
}

+ (NSMutableDictionary *) dictionaryAdvertisementClickWithPlacementCode: (NSString *) placementCode
															channelCode: (NSString *) channelCode
														advertisementId: (NSString *) advertisementId {
	return [self mutableDictionaryWithActionTypeCode:VMAdvertisementClick
										  actionCode:placementCode
											  value1:channelCode
											  value2:nil
											  value3:nil
										redemptionId:nil
											latitude:nil
										   longitude:nil
											 ownerId:nil
											   macId:nil
										  merchantId:nil
										  campaignId:nil
											  itemId:advertisementId
											itemCode:@"A"
											 venueId:nil];
}

#pragma mark - Convert Methods

+ (NSString *) stringWithShareSocialType: (VMShareSocialType) socialType {
	switch (socialType) {
		case VMSEmail:
			return @"E";
		case VMSFacebook:
			return @"FB";
		case VMSTwitter:
			return @"T";
		default:
			break;
	}
}

+ (NSString *) stringWithActivitySocialType: (VMActivitySocialType) socialType {
	switch (socialType) {
		case VMAEmail:
			return @"Email";
		case VMAFacebook:
			return @"Facebook";
		case VMAMobileNumber:
			return @"MobileNumber";
		case VMATwitter:
			return @"Twitter";
		case VMAMSISDN:
			return @"MSISDN";
		default:
			break;
	}
	return nil;
}

+ (NSString *) stringWithTypeGPS: (VMTypeGPS) type {
	switch (type) {
		case VMGPS:
			return @"gps";
			break;
		case VMCell:
			return @"cell";
			break;
		case VMWifi:
			return @"wifi";
			break;
		default:
			break;
	}
	return nil;
}

+ (NSString *) stringWithActionTypeCode: (VMActionTypeCode) actionTypeCode {
	switch (actionTypeCode) {
		case VMButtonClick:
			return @"10";
		case VMButtonClickAnonym:
			return @"10a";
		case VMLocationCheckin:
			return @"1";
		case VMAppPageImpression:
			return @"2";
		case VMAppPageImpressionAnonym:
			return @"2a";
		case VMOfferImpression:
			return @"3";
		case VMOfferRedemption:
			return @"4";
		case VMOfferClickThrough:
			return @"5";
		case VMOfferInstoreRedemption:
			return @"6";
		case VMAppStartup:
			return @"7";
		case VMAppStartupAnonym:
			return @"7a";
		case VMRegistration:
			return @"8";
		case VMLogin:
			return @"9";
		case VMOfferShare:
			return @"11";
		case VMAdvertisementImpression:
			return @"12";
		case VMAdvertisementClick:
			return @"13";
		case VMAppInstall:
			return @"14";
		default:
			break;
	}
	return nil;
}

@end
