//
//  VMConfigurationManager.h
//  VMobSDK
//
//  Created by vmob.co on 7/8/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMRequestManager.h"

@interface VMConfigurationManager : VMRequestManager
@property (nonatomic, strong) NSMutableArray *categories;
/**
 Returns the shared locations manager object for the system.
 
 @return The systemwide locations manager.
 */
+ (instancetype) sharedInstance;

- (NSDictionary *) configuration;

- (VMHTTPRequestOperation *) updateConfigurationWithCompletionBlock: (ObjectBlock) completionBlock
													   failBlock: (ErrorBlock) errorBlock;

#pragma mark - BaseURL API
///--------------------------
/// @name BaseURL Methods
///--------------------------

/**
 Returns Configuration API URL; OR returns locally stored URL for the Configuration API if the configuration parameters have not been loaded from the API yet.
 
 @return Configuration API URL.
 */

+ (NSString *) configurationApiUrl;

/**
 Returns Activity API URL; OR returns locally stored URL for the Activity API if the configuration parameters have not been loaded from the API yet.
 
 @return Activity API URL.
 */

+ (NSString *) activityApiUrl;

/**
 Returns Consumer API URL; OR returns locally stored URL for the Consumer API if the configuration parameters have not been loaded from the API yet.
 
 @return Consumer API URL.
 */

+ (NSString *) consumerApiUrl;

/**
 Returns Location API URL; OR returns locally stored URL for the Location API if the configuration parameters have not been loaded from the API yet.
 
 @return Location API URL.
 */

+ (NSString *) locationApiUrl;

/**
 Returns Offer API URL; OR returns locally stored URL for the Offer API if the configuration parameters have not been loaded from the API yet.
 
 @return Offer API URL.
 */

+ (NSString *) offerApiUrl;

/**
 Returns last loaded configuration file; OR returns nil if the configuration parameters have not been loaded from the API yet.
 
 @return Configuration Dictionary.
 */

+ (NSDictionary *) lastConfiguration;

@end
