//
//  VMConfigurationManager.m
//  VMobSDK
//
//  Created by vmob.co on 7/8/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMConfigurationManager.h"
#import "VMUtils.h"
#import "VMConfigurationRequest.h"
#import "VMCategories.h"

@interface VMConfigurationManager()
@property (nonatomic, strong) NSDictionary *configurationDictionary;
@end

@implementation VMConfigurationManager

- (id)init {
	if (self = [super init]) {
		serviceProvider = [[VMServiceProvider alloc] initWithBaseURL:[NSURL URLWithString:[VMUtils configurationApiUrl]]];
		[self updateConfigurationWithCompletionBlock:nil failBlock:nil];
		self.configurationDictionary = [NSDictionary dictionaryWithContentsOfFile:[VMConfigurationManager configurationPatch]];
	}
	return self;
}

+ (instancetype) sharedInstance {
	static dispatch_once_t once;
    static id sharedInstance;
    dispatch_once(&once, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

- (NSDictionary *) configuration {
	return self.configurationDictionary;
}

#pragma mark - API Methods

- (VMHTTPRequestOperation *) updateConfigurationWithCompletionBlock: (ObjectBlock) completionBlock
													   failBlock: (ErrorBlock) errorBlock {
	VMConfigurationRequest *request = [[VMConfigurationRequest alloc] init];
	
	return [serviceProvider sendRequest:request success:^(VMConfigurationRequest *request) {
		[VMUtils updateConfiguration];
		self.configurationDictionary = request.responseDictionary;
		
		NSArray *array = [self.configurationDictionary objectForKey:@"categories"];
		self.categories = [NSMutableArray array];
		for(NSDictionary *dict in array) {
			[self.categories addObject:[[VMCategories alloc] initWithDictionary:dict]];
		}
		
		[request.responseDictionary writeToFile:[VMConfigurationManager configurationPatch] atomically:YES];
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMConfigurationManager *request, NSError *requestError) {
		self.configurationDictionary = [NSDictionary dictionaryWithContentsOfFile:[VMConfigurationManager configurationPatch]];
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - BaseURL API

+ (NSString *) configurationApiUrl {
	return [VMUtils configurationApiUrl];
}

+ (NSString *) activityApiUrl {
	return [VMUtils activityApiUrl];
}

+ (NSString *) consumerApiUrl {
	return [VMUtils consumerApiUrl];
}

+ (NSString *) locationApiUrl {
	return [VMUtils locationApiUrl];
}

+ (NSString *) offerApiUrl {
	return [VMUtils offerApiUrl];
}

+ (NSDictionary *) lastConfiguration {
	return [NSDictionary dictionaryWithContentsOfFile:[self configurationPatch]];
}

#pragma mark - Private Methods

+ (NSString *) configurationPatch {
	NSString *documentDirectories = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) lastObject];
	return [NSString stringWithFormat:@"%@/%@", documentDirectories, @"VMConfigurationManager"];
}


@end
