//
//  VMConsumer.m
//  VMobSDK
//
//  Created by vmob.co on 5/30/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMConsumerManager.h"
#import "VMRegistrationRequest.h"
#import "VMServiceProvider.h"
#import "VMDetailConsumerRequest.h"
#import "VMUpdateConsumerRequest.h"
#import "VMNewRedeemOfferRequest.h"
#import "VMListRedeemOfferRequest.h"
#import "VMDeleteRedeemOfferRequest.h"
#import "VMLoginRequest.h"
#import "VMListTagRequest.h"
#import "VMAddTagsRequest.h"
#import "VMRemoveTagsRequest.h"
#import "VMCrossReferencesRequest.h"
#import "VMUpdateTagsRequest.h"
#import "VMUtils.h"
#import "VMGiftingCouponsRequest.h"
#import "VMSendGiftCouponRequest.h"
#import "VMDowngradeConsumerToAnonymousRequest.h"

@implementation VMConsumerManager

#pragma mark - Private Methods

- (id)init {
	if (self = [super init]) {
		serviceProvider = [[VMServiceProvider alloc] initWithBaseURL:[NSURL URLWithString:[VMUtils consumerApiUrl]]];
	}
	return self;
}

+ (instancetype) sharedInstance {
	static dispatch_once_t once;
    static id sharedInstance;
    dispatch_once(&once, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

#pragma mark - Publick Methods

- (VMHTTPRequestOperation *) registerConsumerWithMobilePhone: (NSString *) mobilePhoneNumber
													userName: (NSString *) userName
													password: (NSString *) password
												   firstName: (NSString *) firstName
													lastName: (NSString *) lastName
													fullName: (NSString *) fullName
													  gender: (VMGender) gender
												 dateOfBirth: (NSDate *) dateOfBirth
														city: (NSString *) city
											 completionBlock: (ObjectBlock) completionBlock
												   failBlock: (ErrorBlock) errorBlock {
	VMRegistrationRequest *request = [[VMRegistrationRequest alloc] initWithMobilePhone:mobilePhoneNumber
																			   userName:userName
																			   password:password
																			  firstName:firstName
																			   lastName:lastName
																			   fullName:fullName
																				 gender:gender
																			dateOfBirth:dateOfBirth
																				   city:city];
	
	return [serviceProvider sendRequest:request success:^(VMRegistrationRequest *request) {
		_userToken = request.token;
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMRegistrationRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) registerConsumerWithEmail: (NSString *) email
											  userName: (NSString *) userName
											  password: (NSString *) password
											 firstName: (NSString *) firstName
											  lastName: (NSString *) lastName
											  fullName: (NSString *) fullName
												gender: (VMGender) gender
										   dateOfBirth: (NSDate *) dateOfBirth
												  city: (NSString *) city
									   completionBlock: (ObjectBlock) completionBlock
											 failBlock: (ErrorBlock) errorBlock {
	
	VMRegistrationRequest *request = [[VMRegistrationRequest alloc] initWithEmail:email
																		 userName:userName
																		 password:password
																		firstName:firstName
																		 lastName:lastName
																		 fullName:fullName
																		   gender:gender
																	  dateOfBirth:dateOfBirth
																			 city:city];
	return [serviceProvider sendRequest:request success:^(VMRegistrationRequest* request) {
		_userToken = request.token;
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMRegistrationRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) registerConsumerWithAnonymousUserName: (NSString *) userName
														  password: (NSString *) password
												   completionBlock: (ObjectBlock) completionBlock
														 failBlock: (ErrorBlock) errorBlock {
	
	VMRegistrationRequest *request = [[VMRegistrationRequest alloc] initAnonymousWithUserName:userName password:password];
	return [serviceProvider sendRequest:request success:^(VMRegistrationRequest *request) {
		_userToken = request.token;
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMRegistrationRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - Login Methods

- (VMHTTPRequestOperation *) loginConsumerWithUserName: (NSString *) userName
											  password: (NSString *) password
									   completionBlock: (ObjectBlock) completionBlock
											 failBlock: (ErrorBlock) errorBlock {
	
	VMLoginRequest *request = [[VMLoginRequest alloc] initWithUserName:userName password:password];
	return [serviceProvider sendRequest:request success:^(VMLoginRequest *request) {
		_userToken = request.token;
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMRegistrationRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) loginConsumerWithSocialToken: (NSString *) token
											   socialType: (VMSocialType) socialType
										  completionBlock: (ObjectBlock) completionBlock
												failBlock: (ErrorBlock) errorBlock {
	
	VMLoginRequest *request = [[VMLoginRequest alloc] initWithSocialToken:token socialType:socialType];
	return [serviceProvider sendRequest:request success:^(VMLoginRequest *request) {
		_userToken = request.token;
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMRegistrationRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - Manager Methods

- (VMHTTPRequestOperation *) detailConsumerWithUserToken: (NSString *) token
										 completionBlock: (ObjectBlock) completionBlock
											   failBlock: (ErrorBlock) errorBlock {
	
	VMDetailConsumerRequest *request = [[VMDetailConsumerRequest alloc] initWithAccessToken:token];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) updateConsumerWithUserToken: (NSString *) token
												consumer: (VMConsumer *) consumer
										 completionBlock: (ObjectBlock) completionBlock
											   failBlock: (ErrorBlock) errorBlock {
	
	VMUpdateConsumerRequest *request = [[VMUpdateConsumerRequest alloc] initWithAccessToken:token consumer:consumer];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) updateConsumerWithUserToken: (NSString *) token
											   firstName: (NSString *) firstName
												lastName: (NSString *) lastName
												fullName: (NSString *) fullName
												  gender: (VMGender) gender
											 dateOfBirth: (NSDate *) dateOfBirth
													city: (NSString *) city
                                            emailAddress: (NSString *) emailAddress
										 completionBlock: (ObjectBlock) completionBlock
											   failBlock: (ErrorBlock) errorBlock {
	
	VMUpdateConsumerRequest *request = [[VMUpdateConsumerRequest alloc] initWithAccessToken:token
																				  firstName:firstName
																				   lastName:lastName
																				   fullName:fullName
																					 gender:gender
																				dateOfBirth:dateOfBirth
																					   city:city
                                                                               emailAddress:emailAddress];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - RedeemOffers Methods

- (VMHTTPRequestOperation *) createRedeemedOffersWithUserToken: (NSString *) token
													   offerId: (NSString *) offerId
											   completionBlock: (ObjectBlock) completionBlock
													 failBlock: (ErrorBlock) errorBlock {
	
	VMNewRedeemOfferRequest *request = [[VMNewRedeemOfferRequest alloc] initWithAccessToken:token offerId:offerId];
	return [serviceProvider sendRequest:request success:^(VMNewRedeemOfferRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) listRedeemedOffersWithUserToken: (NSString *) token
											   completionBlock: (ObjectBlock) completionBlock
													 failBlock: (ErrorBlock) errorBlock {
	
	VMListRedeemOfferRequest *request = [[VMListRedeemOfferRequest alloc] initWithAccessToken:token];
	return [serviceProvider sendRequest:request success:^(VMListRedeemOfferRequest *request) {
		if (completionBlock) {
			completionBlock(request.redeemOffers);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) deleteRedeemedOffersWithUserToken: (NSString *) token
													   offerId: (NSString *) offerId
											   completionBlock: (ObjectBlock) completionBlock
													 failBlock: (ErrorBlock) errorBlock {
	
	VMDeleteRedeemOfferRequest *request = [[VMDeleteRedeemOfferRequest alloc] initWithAccessToken:token offerId:offerId];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - Tags Methods

- (VMHTTPRequestOperation *) listTagWithUserToken: (NSString *) token
								   completionBlock: (ObjectBlock) completionBlock
										 failBlock: (ErrorBlock) errorBlock {
	VMListTagRequest *request = [[VMListTagRequest alloc] initWithAccessToken:token];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) addTagsWithUserToken: (NSString *) token
								  completionBlock: (ObjectBlock) completionBlock
										failBlock: (ErrorBlock) errorBlock
						   tagValueReferenceCodes: (NSString *) tagValueReferenceCode, ... {
	
	va_list _list;
    va_start(_list, tagValueReferenceCode);
	VMAddTagsRequest *request = [[VMAddTagsRequest alloc] initWithAccessToken:token tagValueReferenceCodes:[VMUtils arraWithInvoke:tagValueReferenceCode withParameters:_list]];
	va_end(_list);
	
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) removeTagsWithUserToken: (NSString *) token
									 completionBlock: (ObjectBlock) completionBlock
										   failBlock: (ErrorBlock) errorBlock
							  tagValueReferenceCodes: (NSString *) tagValueReferenceCode, ...{
	va_list _list;
    va_start(_list, tagValueReferenceCode);
	VMRemoveTagsRequest *request = [[VMRemoveTagsRequest alloc] initWithAccessToken:token tagValueReferenceCodes:[VMUtils arraWithInvoke:tagValueReferenceCode withParameters:_list]];
	va_end(_list);
	
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) updateTagWithUserToken: (NSString *) token
						  tagValueAddReferenceCodes: (NSArray *) tagValueAddReferenceCodes
					   tagValueRemoveReferenceCodes: (NSArray *) tagValueRemoveReferenceCodes
									completionBlock: (ObjectBlock) completionBlock
										  failBlock: (ErrorBlock) errorBlock {
	VMUpdateTagsRequest *request = [[VMUpdateTagsRequest alloc] initWithAccessToken:token tagValueAddReferenceCodes:tagValueAddReferenceCodes tagValueRemoveReferenceCodes:tagValueRemoveReferenceCodes];
	
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - CrossReference Methods

- (VMHTTPRequestOperation *) createCrossReferenceWithPushNotification: (NSString *) token
															userToken: (NSString *) userToken
													  completionBlock: (ObjectBlock) completionBlock
															failBlock: (ErrorBlock) errorBlock {
	return [self createCrossReferenceWithExternalId:token
										  userToken:userToken
										 systemType:VMCrossReferenceApplePushNotification
									completionBlock:completionBlock
										  failBlock:errorBlock];
}

- (VMHTTPRequestOperation *) createCrossReferenceWithExternalId: (NSString *) externalId
													  userToken: (NSString *) userToken
													 systemType: (VMCrossReferenceType) crossReferenceType
												completionBlock: (ObjectBlock) completionBlock
													  failBlock: (ErrorBlock) errorBlock {
	VMCrossReferencesRequest *request = [[VMCrossReferencesRequest alloc] initCrossReferenceWithToken:externalId crossReferenceType:crossReferenceType userToken:userToken];
	return [serviceProvider sendRequest:request success:^(VMBaseRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

#pragma mark - Gifting Coupons

- (VMHTTPRequestOperation *) getGiftingCouponsWithUserToken:(NSString *)token offerId:(NSString *)offerId completionBlock:(ObjectBlock)completionBlock failBlock:(ErrorBlock)errorBlock
{
    VMGiftingCouponsRequest *request = [[VMGiftingCouponsRequest alloc] initWithUserToken:token offerId:offerId];
    
    return [serviceProvider sendRequest:request success:^(VMBaseRequest *request){
        if(completionBlock)
            completionBlock(request.responseDictionary);
    } failure:^(VMBaseRequest *request, NSError *requestError){
        if(errorBlock)
            errorBlock(requestError);
    }];
}

- (VMHTTPRequestOperation *) sendGiftingCouponWithUserToken:(NSString *)token offerId:(NSString *)offerId giftedCopy:(NSString *)giftedCopy giftToFriendsId:(NSArray *)giftToFriendsId completionBlock:(ObjectBlock)completionBlock failBlock:(ErrorBlock)errorBlock
{
    VMSendGiftCouponRequest *request = [[VMSendGiftCouponRequest alloc] initWithUserToken:token offerId:offerId giftedCopy:giftedCopy giftToFriendsId:giftToFriendsId];
    
    return [serviceProvider sendRequest:request success:^(VMBaseRequest *request){
        if(completionBlock)
            completionBlock(request.responseDictionary);
    } failure:^(VMBaseRequest *request, NSError *requestError){
        if(errorBlock)
            errorBlock(requestError);
    }];
}

#pragma mark - Consumer Downgrade To Anonymous

- (VMHTTPRequestOperation *) changeConsumerFacebookAccountToAnonymousWithFBSessionToken:(NSString *)FBtoken completionBlock:(ObjectBlock)completionBlock failBlock:(ErrorBlock)errorBlock
{
    VMDowngradeConsumerToAnonymousRequest *request = [[VMDowngradeConsumerToAnonymousRequest alloc] initConsumerDownToAnonyWithToken:FBtoken];
    
    return [serviceProvider sendRequest:request success:^(VMBaseRequest *request){
        if(completionBlock)
            completionBlock(request.responseDictionary);
    } failure:^(VMBaseRequest *request, NSError *requestError){
        if(errorBlock)
            errorBlock(requestError);
    }];
}

@end
