//
//  VMOfferManager.m
//  VMobSDK
//
//  Created by vmob.co on 6/1/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMOfferManager.h"
#import "VMUtils.h"
#import "VMMerchantsRequest.h"
#import "VMCategoriesOfferRequest.h"
#import "VMNearbyOffersRequest.h"
#import "VMSearchOffersRequest.h"
#import "VMTermsAndConditionsRequest.h"

@implementation VMOfferManager

#pragma mark - Private Methods

- (id)init {
	if (self = [super init]) {
		serviceProvider = [[VMServiceProvider alloc] initWithBaseURL:[NSURL URLWithString:[VMUtils offerApiUrl]]];
	}
	return self;
}

+ (instancetype) sharedInstance {
	static dispatch_once_t once;
    static id sharedInstance;
    dispatch_once(&once, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

#pragma mark - Public Methods

- (VMHTTPRequestOperation *) merchantListWithUserToken: (NSString *) token
											categoryId: (NSString *) categoryId
												offset: (NSString *) offset
												 limit: (NSString *) limit
									   completionBlock: (ObjectBlock) completionBlock
											 failBlock: (ErrorBlock) errorBlock {
	
	VMMerchantsRequest *request = [[VMMerchantsRequest alloc] initWithUserToken:token categoryId:categoryId offset:offset limit:limit];
	return [serviceProvider sendRequest:request success:^(VMMerchantsRequest *request) {
		if (completionBlock) {
			completionBlock(request.merchantList);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) categoriesOfferWithUserToken: (NSString *) token
						includeConsumerExcludedCategories: (BOOL) includeConsumerExcludedCategories
										  completionBlock: (ObjectBlock) completionBlock
												failBlock: (ErrorBlock) errorBlock {
	
	VMCategoriesOfferRequest *request = [[VMCategoriesOfferRequest alloc] initWithUserToken:token
														  includeConsumerExcludedCategories:includeConsumerExcludedCategories];
	
	return [serviceProvider sendRequest:request success:^(VMCategoriesOfferRequest *request) {
		if (completionBlock) {
			completionBlock(request.responseDictionary);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *)nearbyOffersWithUserToken:(NSString *)token
											 latitude:(NSString *)lat
											longitude:(NSString *)lon
										   categoryId:(NSString *)categoryId
											   offset:(NSString *)offset
												limit:(NSString *)limit
											   fields:(NSString *)fields
									  completionBlock:(ObjectBlock)completionBlock
											failBlock:(ErrorBlock)errorBlock {
	
	VMNearbyOffersRequest *request = [[VMNearbyOffersRequest alloc] initWithUserToken:token latitude:lat longitude:lon categoryId:categoryId offset:offset limit:limit fields:fields];
	
	return [serviceProvider sendRequest:request success:^(VMNearbyOffersRequest *request) {
		if (completionBlock) {
			completionBlock(request.offersList);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *)searchOffersWithUserToken:(NSString *)token
										 rankedSearch:(BOOL)ranked
											  keyword:(NSString *)keyword
										   categoryId:(NSString *)categoryId
										   merchantId:(NSString *)merchantId
											   fields:(NSString *)fields
											  orderBy:(NSString *)orderBy
											ascending:(BOOL)ascending
									  completionBlock:(ObjectBlock)completionBlock
											failBlock:(ErrorBlock)errorBlock {
	
	VMSearchOffersRequest *request = [[VMSearchOffersRequest alloc] initWithUserToken:token rankedSearch:ranked keyword:keyword categoryId:categoryId merchantId:merchantId fields:fields orderBy:orderBy ascending:ascending];
	
	return [serviceProvider sendRequest:request success:^(VMSearchOffersRequest *request) {
		if (completionBlock) {
			completionBlock(request.offersList);
		}
	} failure:^(VMBaseRequest * request, NSError *requestError) {
		if (errorBlock) {
			errorBlock(requestError);
		}
	}];
}

- (VMHTTPRequestOperation *) termsAndConditionsWithUserToken: (NSString *) token
                                                     offerId: (NSString *) offerId
                                             completionBlock: (ObjectBlock) completionBlock
                                                   failBlock: (ErrorBlock) errorBlock {
    VMTermsAndConditionsRequest *request = [[VMTermsAndConditionsRequest alloc] initWithUserToken:token offerId:offerId];
    
    return [serviceProvider sendRequest:request toPath:request.path success:^(VMTermsAndConditionsRequest *request) {
        if (completionBlock) {
            completionBlock(request.termsAndConditionsDictionary);
        }
    } failure:^(VMTermsAndConditionsRequest *request, NSError *requestError) {
        if (errorBlock) {
			errorBlock(requestError);
		}
    }];
}

@end
