//
//  RegistrationConsumerRequest.m
//  VMobSDK
//
//  Created by vmob.co on 5/30/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMRegistrationRequest.h"
#import "NSDate+VMAdditions.h"
#import "VMUtils.h"

@interface VMRegistrationRequest()
@property (nonatomic, strong) NSString *path;
@property (nonatomic, strong) NSDictionary *requestDictionary;
@end

@implementation VMRegistrationRequest

#pragma mark - Constructor Methods

- (VMRegistrationRequest *) initWithEmail: (NSString *) email
								 userName: (NSString *) userName
								 password: (NSString *) password
								firstName: (NSString *) firstName
								 lastName: (NSString *) lastName
								 fullName: (NSString *) fullName
								   gender: (VMGender) gender
							  dateOfBirth: (NSDate *) dateOfBirth
									 city: (NSString *) city {
	if (self = [super init]) {
		self.path = @"emailRegistrations";
		self.requestDictionary = @{@"grant_type":@"password",
								@"username": userName,
						  		@"password": password,
						  		@"emailRegistration":
  								@{
									@"emailAddress":email,
		 							@"password":password,
		 							@"firstName":firstName,
		 							@"lastName":lastName,
		 							@"fullName":fullName,
		 							@"gender":[VMUtils stringWithGrenderType:gender],
		 							@"dateOfBirth": [dateOfBirth stringValueWithFormat:@"yyyy-MM-dd"],
		 							@"homeCity":city
		 						}};
	}
	
	return self;
}

- (VMRegistrationRequest *) initWithMobilePhone: (NSString *) mobilePhoneNumber
									   userName: (NSString *) userName
									   password: (NSString *) password
									  firstName: (NSString *) firstName
									   lastName: (NSString *) lastName
									   fullName: (NSString *) fullName
										 gender: (VMGender) gender
									dateOfBirth: (NSDate *) dateOfBirth
										   city: (NSString *) city {
	if (self = [super init]) {
		self.path = @"mobilePhoneNumberRegistrations";
		self.requestDictionary = @{
						  	@"grant_type":@"password",
							@"username": userName,
							@"password": password,
							@"mobilePhoneNumberRegistration":
							@{
								@"mobilePhoneNumber":mobilePhoneNumber,
								@"password":password,
								@"firstName":firstName,
								@"lastName":lastName,
								@"fullName":fullName,
								@"gender":[VMUtils stringWithGrenderType:gender],
								@"dateOfBirth": [dateOfBirth stringValueWithFormat:@"yyyy-MM-dd"],
								@"homeCity":city
							}};
	}
	return self;
}

- (VMRegistrationRequest *) initAnonymousWithUserName: (NSString *) userName
											 password: (NSString *) password {
	if (self = [super init]) {
		self.path = @"anonymousRegistrations";
		self.requestDictionary = @{
						  		@"grant_type":@"password",
								@"username": userName,
								@"password": password};
	}
	return self;
}

@end