//
//  VMServiceProvider.m
//  VMobSDK
//
//  Created by vmob.co on 5/29/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMServiceProvider.h"
#import "VMBaseRequest.h"
#import "NSString+VMValidation.h"
#import "NSDictionary+VMValidation.h"
#import "NSString+VMToolkitAdditions.h"

@implementation VMServiceProvider

- (VMHTTPRequestOperation *) createOperationWithRequest:(VMBaseRequest *) serviceRequest toPath:(NSString *) path success:(RequestSuccessCallback) success failure:(RequestFailCallback) failure {
	NSDictionary *requestDict = [serviceRequest requestDictionary];
	
	NSMutableURLRequest *request = [self requestWithMethod:[serviceRequest method]
													  path:[self addQueryParamWithBaseRequest:serviceRequest path:path]
												parameters:requestDict];
	
	request = [self addHeaderWithRequest:request baseRequest:serviceRequest];
	VMHTTPRequestOperation *operation = [super HTTPRequestOperationWithRequest:request
																serviceRequest:serviceRequest success:success
																	   failure:failure];
	operation.serviceRequest = serviceRequest;
	return operation;
}

- (VMHTTPRequestOperation *) sendRequest:(VMBaseRequest *) serviceRequest toPath:(NSString *) path success:(RequestSuccessCallback) success failure:(RequestFailCallback) failure {
	VMHTTPRequestOperation *operation = [self createOperationWithRequest:serviceRequest toPath:path success:success failure:failure];
	[self enqueueHTTPRequestOperation:operation];
	return operation;
}

- (NSMutableURLRequest *) addHeaderWithRequest: (NSMutableURLRequest *) request baseRequest: (VMBaseRequest *) baseRequest {
	NSDictionary *header = [baseRequest dictionaryHeader];
	for(NSString *key in [header allKeys]) {
		[request addValue:[header objectForKey:key] forHTTPHeaderField:key];
	}
	return request;
}

- (NSString *) addQueryParamWithBaseRequest: (VMBaseRequest *) baseRequest path:(NSString *) path{
	NSDictionary *queryParam = [baseRequest dictionaryQueryParam];
	if ([queryParam count] < 1) {
		return path;
	}
	path = [NSString stringWithFormat:@"%@?",path];
	for(NSString *key in [queryParam allKeys]) {
		path = [NSString stringWithFormat:@"%@%@=%@&", path, key, [queryParam objectForKey:key]];
	}
	return [path substringToIndex:path.length - 1];
}

@end
