//
//  UpdateConsumerRequest.m
//  VMobSDK
//
//  Created by vmob.co on 5/31/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMUpdateConsumerRequest.h"
#import "NSDate+VMAdditions.h"
#import "NSDictionary+VMValidation.h"
#import "VMUtils.h"

@interface VMUpdateConsumerRequest()
@property (nonatomic, strong) NSMutableDictionary *dictionaryHeader;
@property (nonatomic, strong) NSMutableDictionary *requestDictionary;
@end

@implementation VMUpdateConsumerRequest

- (VMUpdateConsumerRequest *) initWithAccessToken: (NSString *) token
									   consumer: (VMConsumer *) consumer {
	return [self initWithAccessToken:token
						   firstName:consumer.firstName
							lastName:consumer.lastName
							fullName:consumer.fullName
							  gender:consumer.gender
						 dateOfBirth:consumer.dateOfBirth
								city:consumer.homeCity
                        emailAddress:consumer.emailAddress];
}

- (VMUpdateConsumerRequest *) initWithAccessToken: (NSString *) token
                                        firstName: (NSString *) firstName
                                         lastName: (NSString *) lastName
                                         fullName: (NSString *) fullName
                                           gender: (VMGender) gender
                                      dateOfBirth: (NSDate *) dateOfBirth
                                             city: (NSString *) city
                                     emailAddress: (NSString *) emailAddress {
	if (self = [super init]) {
		self.dictionaryHeader = [super dictionaryHeader];
		[self.dictionaryHeader setObjectOrDoNothing:token forKey:@"Authorization"];
		
		self.requestDictionary = [NSMutableDictionary new];
		[self.requestDictionary setObjectOrDoNothing:firstName forKey:@"firstName"];
		[self.requestDictionary setObjectOrDoNothing:lastName forKey:@"lastName"];
		[self.requestDictionary setObjectOrDoNothing:fullName forKey:@"fullName"];
		[self.requestDictionary setObjectOrDoNothing:[VMUtils stringWithGrenderType:gender] forKey:@"gender"];
		[self.requestDictionary setObjectOrDoNothing:[dateOfBirth stringValueWithFormat:@"yyyy-MM-dd"] forKey:@"dateOfBirth"];
		[self.requestDictionary setObjectOrDoNothing:city forKey:@"homeCity"];
        [self.requestDictionary setObjectOrDoNothing:emailAddress forKey:@"emailAddress"];
	}
	return self;
}

- (NSString *) method {
	return @"PUT";
}

- (NSString *) path {
	return @"consumers";
}

@end
