//
//  Utils.m
//  VMobSDK
//
//  Created by vmob.co on 5/29/13.
//  Copyright (c) 2013 vmob.co. All rights reserved.
//

#import "VMUtils.h"
#import "VMReachability.h"
#import "VMConfigurationManager.h"
#import <CoreTelephony/CTTelephonyNetworkInfo.h>
#import <CoreTelephony/CTCarrier.h>
#import "NSDictionary+VMValidation.h"
#include <sys/socket.h> // Per msqr
#include <sys/sysctl.h>
#include <net/if.h>
#include <net/if_dl.h>
#import <CommonCrypto/CommonDigest.h>
#import <UIKit/UIKit.h>
#import "VMKeychain.h"

#define kBundleName @"VMobSDKBundle"
#define kConfigurationPlist @"VMobSDKConfiguration"
#define kConfigurationInfoPlist @"VMobSDKConfiguration"
//URL
#define kConfigurationApiUrl @"configurationApiUrl"
#define kActivityApiUrl @"activityApiUrl"
#define kConsumerApiUrl @"consumerApiUrl"
#define kLocationApiUrl @"locationApiUrl"
#define kOfferApiUrl @"offerApiUrl"
//key
#define kAuthorizationKey @"AuthorizationKey"
#define kSiteID @"Site_ID"
#define kBuildVersion @"Build versions"
#define kPrintLog @"PrintLog"
//activity
#define kActivitiesBatchSize @"activitiesBatchSize"
#define kActivitiesSendMinTimerInSeconds @"activitiesSendMinTimerInSeconds"
#define kActivitiesExpireSize @"activitiesExpireSize"
#define kActivitiesExpireAgeInDays @"activitiesExpireAgeInDays"
//location
#define kUpdateLocationTimeInterval @"gpsPollIntervalInSeconds"
#define kGpsBackgroundMinIntervalInSeconds @"gpsBackgroundMinIntervalInSeconds"
#define kLocationAccuracy @"LocationAccuracy"
#define kLocationDistanceFilter @"LocationDistanceFilter"
//Configuration
#define kConfigurationCheckInterval @"configurationCheckInterval"
#define kAddCodeForAutoConfgurationUpdate @"addCodeForAutoConfgurationUpdate"

@interface VMUtils()
@end

@implementation VMUtils

+ (instancetype) sharedInstance {
	static dispatch_once_t once;
    static id sharedInstance;
    dispatch_once(&once, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

#pragma mark - Header Methods

+ (NSString *) currentLatitude {
	if ([VMUtils sharedInstance].currentLocation) {
		return [NSString stringWithFormat:@"%f",[VMUtils sharedInstance].currentLocation.coordinate.latitude];
	}
	else {
		return nil;
	}
}

+ (NSString *) currentLongitude {
	if ([VMUtils sharedInstance].currentLocation) {
		return [NSString stringWithFormat:@"%f",[VMUtils sharedInstance].currentLocation.coordinate.longitude];
	}
	else {
		return nil;
	}
}

+ (NSString *) currentAccuracy {
	if ([VMUtils sharedInstance].currentLocation) {
		return [NSString stringWithFormat:@"%f",[VMUtils sharedInstance].currentLocation.horizontalAccuracy];
	}
	else {
		return nil;
	}
}

+ (NSString *) deviceType {
	if (kIsPad) {
		return @"i_t";
	}
	else {
		return @"i_p";
	}
}

+ (NSString *) applicationVersion {
	return [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
}

+ (NSString *) deviceScreenResolution {
	return [NSString stringWithFormat:@"%0.fx%0.f", [UIScreen mainScreen].bounds.size.height * UIScreen.mainScreen.scale,[UIScreen mainScreen].bounds.size.width * UIScreen.mainScreen.scale];
}

+ (NSString *) carrierName {
	CTTelephonyNetworkInfo *networkInfo = [[CTTelephonyNetworkInfo alloc] init];
	CTCarrier *carrier = [networkInfo subscriberCellularProvider];
	return carrier.carrierName;
}

+ (NSString *) typeNetworkConnection {
	switch ([[VMReachability reachabilityForLocalWiFi] currentReachabilityStatus]) {
		case ReachableViaWWAN:
			return @"3g";
		case ReachableViaWiFi:
			return @"wifi";
		default:
			return @"none";
	}
}

+ (NSString *) offsetUTC {
	NSInteger time = [[NSTimeZone systemTimeZone] secondsFromGMT];
	NSString *timeString = [VMUtils stringFromTimeInterval:ABS(time)];
	if (time < 0) {
		return [NSString stringWithFormat:@"-%@",timeString];
	}
	else {
		return [NSString stringWithFormat:@"+%@",timeString];
	}
}

#pragma mark - Private Methods

+ (NSString *)stringFromTimeInterval:(NSInteger) interval {
    NSInteger minutes = (interval / 60) % 60;
    NSInteger hours = (interval / 3600);
    return [NSString stringWithFormat:@"%02i:%02i", hours, minutes];
}

+ (NSDictionary *) configurationDictionary {
	NSString *bundlePath = [[NSBundle mainBundle] pathForResource:kBundleName ofType:@"bundle"];
	NSString *file = [[NSBundle bundleWithPath:bundlePath] pathForResource:kConfigurationPlist ofType:@"plist"];
	return [NSDictionary dictionaryWithContentsOfFile:file];
}

+ (NSDictionary *) configurationDictionaryInfoPlist {
	return [[[NSBundle mainBundle] infoDictionary] objectForKey:kConfigurationInfoPlist];
}

#pragma mark - BaseURL API

+ (NSString *) urlSetSiteID: (NSString *) url {
	return [NSString stringWithFormat:url, [[VMUtils configurationDictionaryInfoPlist] objectForKey:kSiteID]];
}

+ (NSString *) urlConfiguration: (NSString *) key {
	NSString *result = [[[VMConfigurationManager sharedInstance] configuration] objectForKey:key];
	if (result) {
		return result;
	}
	else {
		return [VMUtils urlSetSiteID:[[VMUtils configurationDictionary] objectForKey:key]];
	}
}

+ (NSString *) configurationApiUrl {
	NSString *result = [[VMConfigurationManager lastConfiguration] objectForKey:kConfigurationApiUrl];
	if (result) {
		return result;
	}
	else {
		return [VMUtils urlSetSiteID:[[VMUtils configurationDictionary] objectForKey:kConfigurationApiUrl]];
	}
}

+ (NSString *) activityApiUrl {
	return [self urlConfiguration:kActivityApiUrl];
}

+ (NSString *) consumerApiUrl {
	return [self urlConfiguration:kConsumerApiUrl];
}

+ (NSString *) locationApiUrl {
	return [self urlConfiguration:kLocationApiUrl];
}

+ (NSString *) offerApiUrl {
	return [self urlConfiguration:kOfferApiUrl];
}

#pragma mark - User Constant

+ (BOOL) printLog {
	return [[self configurationDictionary] boolForKey:kPrintLog];
}

+ (NSString *) buildVersion {
	return [[self configurationDictionary] objectForKey:kBuildVersion];
}

+ (NSString *) uniqueGlobalDeviceIdentifier{
	if (IsIOS7) {
		return [self uuid];
	}
	else {
		NSString *macaddress = [self macaddress];
		NSString *uniqueIdentifier = [self stringFromMD5:macaddress];
		[UIDevice currentDevice];
		return uniqueIdentifier;
	}
}

+ (NSString *) authorizationKey {
	return [[VMUtils configurationDictionaryInfoPlist] objectForKey:kAuthorizationKey];
}


+ (NSObject *) validConfigurationKey: (NSString *) key {
	NSNumber * object = [[[VMConfigurationManager sharedInstance] configuration] objectForKey:key];
	if (object) {
		return object;
	}
	object = [[VMUtils configurationDictionary] objectForKey:key];
	if (object) {
		return object;
	}
	return nil;
}

//Activity Manager
+ (NSTimeInterval) activitiesSendMinTimerInSeconds {
	NSNumber *interval= (NSNumber *)[self validConfigurationKey:kActivitiesSendMinTimerInSeconds];
	return [interval integerValue];
}

+ (NSInteger) activitiesBatchSize {
	NSNumber *size = (NSNumber *)[self validConfigurationKey:kActivitiesBatchSize];
	return [size integerValue];
}

+ (NSUInteger) activitiesExpireSize {
	NSNumber *size = (NSNumber *)[self validConfigurationKey:kActivitiesExpireSize];
	return [size integerValue];
}

+ (NSTimeInterval) activitiesExpireAgeInSeconds {
	NSInteger interval= ([(NSNumber *)[self validConfigurationKey:kActivitiesExpireAgeInDays] integerValue]) *24*60*60;//Days to seconds
	return interval;
}

//Location Manager
+ (NSTimeInterval) updateLocationTimeInterval {
	NSNumber *result = ((NSNumber *)[self validConfigurationKey:kUpdateLocationTimeInterval]);
	if (result) {
		return [result integerValue];
	}
	else {
		return [[[VMUtils configurationDictionaryInfoPlist] objectForKey:kUpdateLocationTimeInterval] integerValue];
	}
}

//gpsBackgroundMinIntervalInSeconds
+ (NSTimeInterval) gpsBackgroundMinIntervalInSeconds {
	NSNumber *result = ((NSNumber *)[self validConfigurationKey:kGpsBackgroundMinIntervalInSeconds]);
	if (result) {
		return [result integerValue];
	}
	else {
		return [[[VMUtils configurationDictionaryInfoPlist] objectForKey:kGpsBackgroundMinIntervalInSeconds] integerValue];
	}
}

+ (double) locationAccuracy {
	NSNumber *result = ((NSNumber *)[self validConfigurationKey:kLocationAccuracy]);
	if (result) {
		return [result doubleValue];
	}
	else {
		return [[[VMUtils configurationDictionaryInfoPlist] objectForKey:kLocationAccuracy] doubleValue];
	}
}

+ (double) locationDistanceFilter {
	NSNumber *result = ((NSNumber *)[self validConfigurationKey:kLocationDistanceFilter]);
	if (result) {
		return [result doubleValue];
	}
	else {
		return [[[VMUtils configurationDictionaryInfoPlist] objectForKey:kLocationDistanceFilter] doubleValue];
	}
}

#pragma mark - Configuration Methods

+ (BOOL) addCodeForAutoConfgurationUpdate
{
    return [[VMUtils configurationDictionaryInfoPlist] boolForKey:kAddCodeForAutoConfgurationUpdate];
}

+ (BOOL) isUpdateConfiguration {
	NSNumber *configurationCheckInterval = [[VMUtils configurationDictionaryInfoPlist] objectForKey:kConfigurationCheckInterval];
	if (configurationCheckInterval && [configurationCheckInterval integerValue] > 0) {
		NSDate *lastDate = [self lastUpdateConfiguration];
		if (lastDate) {
			NSTimeInterval interval = [[NSDate date] timeIntervalSinceDate:lastDate];
			return ([configurationCheckInterval integerValue] < interval);
		}
		else {
			return YES;
		}
	}
	return NO;
}

+ (NSDate *) lastUpdateConfiguration {
	return [[NSUserDefaults standardUserDefaults] objectForKey:kConfigurationCheckInterval];
}

+ (void) updateConfiguration {
	[[NSUserDefaults standardUserDefaults] setObject:[NSDate date] forKey:kConfigurationCheckInterval];
}

#pragma mark - Other Methods

+ (void) setCLLocation: (CLLocation *) location {
	[VMUtils sharedInstance].currentLocation = location;
}

+ (NSMutableArray *) arraWithInvoke:(NSObject *) object withParameters: (va_list) args {
	NSMutableArray *list = [NSMutableArray array];
	
	for (NSObject *obj = object; obj != nil; obj = va_arg(args, NSObject*)) {
		[list addObject:obj];
	}
	
	if([list count] > 0)
		return list;
	return nil;
}

+ (VMGender) grenderTypeWithString: (NSString *) string {
	if ([string isEqualToString:@"m"]) {
		return VMMale;
	}
	else {
		return VMFemale;
	}
}

+ (NSString *) stringWithBOOL: (BOOL) value {
	if (value) {
		return @"true";
	}
	else {
		return @"false";
	}
}

+ (NSString *) stringWithGrenderType: (VMGender) gender {
	switch (gender) {
		case VMMale:
			return @"m";
			break;
		case VMFemale:
			return @"f";
		default:
			break;
	}
}

+ (NSString *) stringWithSocialToken:(VMSocialType) socialType {
	switch (socialType) {
		case VMFacebook:
			return @"facebookOAuth2";
			break;
        case VMFacebookAT:
            return @"facebookAccessToken";
            break;
		case VMTwitter:
			return @"twitterOAuth2";
			break;
		default:
			break;
	}
}

+ (NSDateFormatter *) UTCFormater {
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSTimeZone *timeZone = [NSTimeZone timeZoneWithName:@"UTC"];
    [dateFormatter setTimeZone:timeZone];
    [dateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss'Z'"];
	return dateFormatter;
}

+ (NSString *)UTCFormateDate:(NSDate *)date {
    NSString *dateString = [[self UTCFormater] stringFromDate:date];
    return dateString;
}

+ (NSDate *) dateUTCString: (NSString *) date {
	return [[self UTCFormater] dateFromString:date];
}

#pragma mark - UIDID

+ (NSString *) createUUID {
    NSString *uuid = [[[UIDevice currentDevice] identifierForVendor] UUIDString];
    [VMKeychain setPassword:uuid forService:kBundleIdentifier account:@"user"];
    return uuid;
}

+ (NSString*) uuid {
    NSString* uuid = [VMKeychain passwordForService:kBundleIdentifier account:@"user"];
    if(!uuid){
        uuid = [self createUUID];
    }
    return uuid;
}

+ (NSString *) macaddress{
    
    int                 mib[6];
    size_t              len;
    char                *buf;
    unsigned char       *ptr;
    struct if_msghdr    *ifm;
    struct sockaddr_dl  *sdl;
    
    mib[0] = CTL_NET;
    mib[1] = AF_ROUTE;
    mib[2] = 0;
    mib[3] = AF_LINK;
    mib[4] = NET_RT_IFLIST;
    
    if ((mib[5] = if_nametoindex("en0")) == 0) {
        printf("Error: if_nametoindex error\n");
        return NULL;
    }
    
    if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
        printf("Error: sysctl, take 1\n");
        return NULL;
    }
    
    if ((buf = malloc(len)) == NULL) {
        printf("Could not allocate memory. error!\n");
        return NULL;
    }
    
    if (sysctl(mib, 6, buf, &len, NULL, 0) < 0) {
        printf("Error: sysctl, take 2");
        free(buf);
        return NULL;
    }
    
    ifm = (struct if_msghdr *)buf;
    sdl = (struct sockaddr_dl *)(ifm + 1);
    ptr = (unsigned char *)LLADDR(sdl);
    NSString *outstring = [NSString stringWithFormat:@"%02X:%02X:%02X:%02X:%02X:%02X",
                           *ptr, *(ptr+1), *(ptr+2), *(ptr+3), *(ptr+4), *(ptr+5)];
    free(buf);
    
    return outstring;
}

+ (NSString *) stringFromMD5: (NSString *)val{
    
    if(val == nil || [val length] == 0)
        return nil;
    
    const char *value = [val UTF8String];
    
    unsigned char outputBuffer[CC_MD5_DIGEST_LENGTH];
    CC_MD5(value, strlen(value), outputBuffer);
    
    NSMutableString *outputString = [[NSMutableString alloc] initWithCapacity:CC_MD5_DIGEST_LENGTH * 2];
    for(NSInteger count = 0; count < CC_MD5_DIGEST_LENGTH; count++){
        [outputString appendFormat:@"%02x",outputBuffer[count]];
    }
    
    return outputString;
}
@end
